<?php
if (!function_exists('GetRemotePageContents')) {
	
	$error_cumulator = "";
	
	//
	// Retrieves remote page by using any available method
	//
	function GetRemotePageContents($url, $proxy_ip="", $proxy_port="", $force_method = 0) {
		
		$escapedurl = str_replace(" ","+",str_replace("&#039;","'",$url));
		
		$remotecontents = array("Contents"=>"", "Method"=>"", "Errors" => "");
		
		// Check if we can use file_get_contents on this server
		if (function_exists("curl_init") && ($force_method<2)) {
			$remotecontents["Contents"] = GetRemotePageWithCurl($escapedurl, $proxy_ip, $proxy_port);
			$remotecontents["Method"] = "cUrl";
		} else {
			if (ini_get('allow_url_fopen') == '1' && ($force_method == 0 || $force_method == 2)) {
				$remotecontents["Contents"] = GetRemotePageWithFContents($escapedurl, $proxy_ip, $proxy_port);
				$remotecontents["Method"] = "GetFileContents";
			} else {
				$remotecontents["Contents"] = GetRemotePageWithFRead($escapedurl, $proxy_ip, $proxy_port);
				$remotecontents["Method"] = "FPuts";
			}
		}
		$remotecontents["Errors"] = $error_cumulator;
		return $remotecontents;
	}
	
	
	
	//
	// Retrieves remote page by using cUrl
	//
	function GetRemotePageWithCurl($url, $proxy_ip, $proxy_port) {
		global $error_cumulator;
		set_time_limit(60); // Just to be on safe side - allow it to work for 60 seconds
		// initialize a new curl resource
		$channel = curl_init();
		
		// set the url to fetch
		curl_setopt($channel, CURLOPT_URL, $url);
		
		// don't give me the headers just the content
		curl_setopt($channel, CURLOPT_HEADER, false);
		
		// We need that data DESPERATE!!!!
		curl_setopt($channel, CURLOPT_CONNECTTIMEOUT, 10);
		curl_setopt($channel, CURLOPT_TIMEOUT, 60);
		
		// return the value instead of printing the response to browser
		curl_setopt($channel, CURLOPT_RETURNTRANSFER, true);
		
		// If Proxy must be uset - set options for that
		if (($proxy_ip != "") && ($proxy_port != "")) {
			//curl_setopt ($channel, CURLOPT_HTTPPROXYTUNNEL, TRUE);
			curl_setopt ($channel, CURLOPT_PROXYTYPE, CURLPROXY_HTTP);
			curl_setopt ($channel, CURLOPT_PROXY, $proxy_ip.":".$proxy_port);
			curl_setopt ($channel, CURLOPT_SSL_VERIFYPEER, FALSE);
		}
		
		// use a user agent to mimic a browser
		curl_setopt($channel, CURLOPT_USERAGENT, "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-GB; rv:1.9.1.2) Gecko/20090729 Firefox/3.5.2");
		
		$htmlz = curl_exec($channel);
		// Accumulate error if happened
		if (curl_errno($channel)) {
			$error_cumulator .= "cUrl Error#".curl_errno($channel).": ".curl_error($channel)."<br/>";
		}
		// always close the session and free all resources
		curl_close($channel); 
		return $htmlz;
	}
	
	
	
	
	//
	// Retrieves remote page by using get_file_contents()
	//
	function GetRemotePageWithFContents($url, $proxy_ip, $proxy_port) {
		set_time_limit(120); // Just to be on safe side - allow it to work for 120 seconds
		global $error_cumulator;
		if (($proxy_ip != "") && ($proxy_port != ""))
		{
			$context = array (
					'http' => array (
						'proxy' => $proxy_ip.":".$proxy_port, 
						'request_fulluri' => true,
						'user_agent'=>'User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1)',
						'timeout' => 120,
						)
					);
		}
		else 
		{
			$context = array (
					'http' => array (
						'request_fulluri' => true,
						'user_agent'=>'User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1)'
						)
					);
		}
		$context = stream_context_create ($context); 
		$htmlz = @file_get_contents($url,$context);
		if (!$htmlz) {
			if (!$http_response_header) {
				$error_cumulator .= "Bad URL or timeout on FGet<br/>";
			} else {
				$nLines = count( $http_response_header );
				for ( $i = $nLines-1; $i >= 0; $i-- )
				{
					$line = $http_response_header[$i];
					if ( strncasecmp( "HTTP", $line, 4 ) == 0 )
					{
						$response = explode( ' ', $line );
						$error_cumulator .= "FGet returned HTTP error ".$response[1]."<br/>";
						break;
					}
				}
			}
		}
		return $htmlz;
	}
	
	
	//
	// Retrieves remote page by using fputs
	//
	function GetRemotePageWithFRead($url, $proxy_ip, $proxy_port) {
		// get the host name and url path
		global $error_cumulator;
		$parsedUrl = parse_url($url);
		$host = $parsedUrl['host'];
		if (isset($parsedUrl['path'])) {
			$path = $parsedUrl['path'];
		} else {
			// the url is pointing to the host like http://www.mysite.com
			$path = '/';
		}
		
		if (isset($parsedUrl['query'])) {
			$path .= '?' . $parsedUrl['query'];
		}
		
		if (isset($parsedUrl['port'])) {
			$port = $parsedUrl['port'];
		} else {
			// most sites use port 80
			$port = '80';
		}
		
		$timeout = 120;
		$response = '';
		
		// connect to the remote server
		if (($proxy_ip != "") && ($proxy_port != "")) {
			$fp = @fsockopen($proxy_ip, $proxy_port, $errno, $errstr, $timeout );
		} else {
			$fp = @fsockopen($host, $port, $errno, $errstr, $timeout );
		}
		if( !$fp ) {
			$error_cumulator .= "fSockOpen error#".$errno.": ".$errstr."</br>";
			return false;
		} else {
			// send the necessary headers to get the file
			fputs($fp, "GET http://".$host.$path." HTTP/1.0\r\n" .
					"Host: ".$host."\r\n" .
					"User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.2) Gecko/20070219 Firefox/2.0.0.2\r\n" .
					"Accept: */*\r\n" .
					"Accept-Language: en-us,en;q=0.5\r\n" .
					"Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7\r\n" .
					"Keep-Alive: 300\r\n" .
					"Connection: keep-alive\r\n" .
					"Referer: http://".$host."\r\n\r\n");
			
			// retrieve the response from the remote server
			while ( $line = fread( $fp, 4096 ) ) {
				$response .= $line;
			}
			fclose($fp);
			
			// strip the headers
			$pos      = strpos($response, "\r\n\r\n");
			$header   = substr($response, 0, $pos);
			$nLines = count( $header );
			for ( $i = $nLines-1; $i >= 0; $i-- )
			{
				$line = $header[$i];
				if ( strncasecmp( "HTTP", $line, 4 ) == 0 )
				{
					$response = explode( ' ', $line );
					if ($response[1] != "200") {
						$error_cumulator .= "fputs returned HTTP error ".$response[1]."<br/>";
					}
					break;
				}
			}
			
			$response = substr($response, $pos + 4);
		}
		
		// return the file content
		return $response;
	}
	
}
?>